# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
from enum import Enum


class DeviceType(Enum):
    """
    Denotes type of an device and units it accepts.
    """

    UNKNOWN = 0
    LINEAR = 1
    ROTARY = 2
