# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #


class SettingConstants:

    """Accel."""
    ACCEL = "accel"

    """Calibration Type."""
    CALIBRATION_TYPE = "calibration.type"

    """Cloop Continuous Enable."""
    CLOOP_CONTINUOUS_ENABLE = "cloop.continuous.enable"

    """Cloop Counts."""
    CLOOP_COUNTS = "cloop.counts"

    """Cloop Displace Tolerance."""
    CLOOP_DISPLACE_TOLERANCE = "cloop.displace.tolerance"

    """Cloop Duration Max."""
    CLOOP_DURATION_MAX = "cloop.duration.max"

    """Cloop Enable."""
    CLOOP_ENABLE = "cloop.enable"

    """Cloop Mode."""
    CLOOP_MODE = "cloop.mode"

    """Cloop Recovery Enable."""
    CLOOP_RECOVERY_ENABLE = "cloop.recovery.enable"

    """Cloop Settle Period."""
    CLOOP_SETTLE_PERIOD = "cloop.settle.period"

    """Cloop Settle Tolerance."""
    CLOOP_SETTLE_TOLERANCE = "cloop.settle.tolerance"

    """Cloop Stalltimeout."""
    CLOOP_STALLTIMEOUT = "cloop.stalltimeout"

    """Cloop Steps."""
    CLOOP_STEPS = "cloop.steps"

    """Cloop Timeout."""
    CLOOP_TIMEOUT = "cloop.timeout"

    """Comm Address."""
    COMM_ADDRESS = "comm.address"

    """Comm Alert."""
    COMM_ALERT = "comm.alert"

    """Comm Checksum."""
    COMM_CHECKSUM = "comm.checksum"

    """Comm Protocol."""
    COMM_PROTOCOL = "comm.protocol"

    """Comm Rs 232 Baud."""
    COMM_RS_232_BAUD = "comm.rs232.baud"

    """Comm Rs 232 Protocol."""
    COMM_RS_232_PROTOCOL = "comm.rs232.protocol"

    """Comm Rs 485 Baud."""
    COMM_RS_485_BAUD = "comm.rs485.baud"

    """Comm Rs 485 Enable."""
    COMM_RS_485_ENABLE = "comm.rs485.enable"

    """Comm Rs 485 Protocol."""
    COMM_RS_485_PROTOCOL = "comm.rs485.protocol"

    """Comm Usb Protocol."""
    COMM_USB_PROTOCOL = "comm.usb.protocol"

    """Deviceid."""
    DEVICEID = "deviceid"

    """Driver Current Hold."""
    DRIVER_CURRENT_HOLD = "driver.current.hold"

    """Driver Current Max."""
    DRIVER_CURRENT_MAX = "driver.current.max"

    """Driver Current Run."""
    DRIVER_CURRENT_RUN = "driver.current.run"

    """Driver Current Servo."""
    DRIVER_CURRENT_SERVO = "driver.current.servo"

    """Driver Dir."""
    DRIVER_DIR = "driver.dir"

    """Driver Enabled."""
    DRIVER_ENABLED = "driver.enabled"

    """Driver Temperature."""
    DRIVER_TEMPERATURE = "driver.temperature"

    """Encoder Count."""
    ENCODER_COUNT = "encoder.count"

    """Encoder Count Cal."""
    ENCODER_COUNT_CAL = "encoder.count.cal"

    """Encoder Count Calibrated."""
    ENCODER_COUNT_CALIBRATED = "encoder.count.calibrated"

    """Encoder Counth."""
    ENCODER_COUNTH = "encoder.counth"

    """Encoder Dir."""
    ENCODER_DIR = "encoder.dir"

    """Encoder Error."""
    ENCODER_ERROR = "encoder.error"

    """Encoder Fault Type."""
    ENCODER_FAULT_TYPE = "encoder.fault.type"

    """Encoder Filter."""
    ENCODER_FILTER = "encoder.filter"

    """Encoder Index Count."""
    ENCODER_INDEX_COUNT = "encoder.index.count"

    """Encoder Index Mode."""
    ENCODER_INDEX_MODE = "encoder.index.mode"

    """Encoder Index Phase."""
    ENCODER_INDEX_PHASE = "encoder.index.phase"

    """Encoder Mode."""
    ENCODER_MODE = "encoder.mode"

    """Encoder Pos."""
    ENCODER_POS = "encoder.pos"

    """Encoder Pos Error."""
    ENCODER_POS_ERROR = "encoder.pos.error"

    """Filter Holderid."""
    FILTER_HOLDERID = "filter.holderid"

    """Force Average."""
    FORCE_AVERAGE = "force.average"

    """Joy Debug."""
    JOY_DEBUG = "joy.debug"

    """Knob Dir."""
    KNOB_DIR = "knob.dir"

    """Knob Distance."""
    KNOB_DISTANCE = "knob.distance"

    """Knob Enable."""
    KNOB_ENABLE = "knob.enable"

    """Knob Force."""
    KNOB_FORCE = "knob.force"

    """Knob Forceprofile."""
    KNOB_FORCEPROFILE = "knob.forceprofile"

    """Knob Maxspeed."""
    KNOB_MAXSPEED = "knob.maxspeed"

    """Knob Mode."""
    KNOB_MODE = "knob.mode"

    """Knob Speedprofile."""
    KNOB_SPEEDPROFILE = "knob.speedprofile"

    """Lamp Current."""
    LAMP_CURRENT = "lamp.current"

    """Lamp Current Max."""
    LAMP_CURRENT_MAX = "lamp.current.max"

    """Lamp Flux."""
    LAMP_FLUX = "lamp.flux"

    """Lamp Flux Max."""
    LAMP_FLUX_MAX = "lamp.flux.max"

    """Lamp Status."""
    LAMP_STATUS = "lamp.status"

    """Lamp Temperature."""
    LAMP_TEMPERATURE = "lamp.temperature"

    """Lamp Wavelength Fwhm."""
    LAMP_WAVELENGTH_FWHM = "lamp.wavelength.fwhm"

    """Lamp Wavelength Peak."""
    LAMP_WAVELENGTH_PEAK = "lamp.wavelength.peak"

    """Limit Approach Accel."""
    LIMIT_APPROACH_ACCEL = "limit.approach.accel"

    """Limit Approach Maxspeed."""
    LIMIT_APPROACH_MAXSPEED = "limit.approach.maxspeed"

    """Limit Away Action."""
    LIMIT_AWAY_ACTION = "limit.away.action"

    """Limit Away Edge."""
    LIMIT_AWAY_EDGE = "limit.away.edge"

    """Limit Away Pos."""
    LIMIT_AWAY_POS = "limit.away.pos"

    """Limit Away Posupdate."""
    LIMIT_AWAY_POSUPDATE = "limit.away.posupdate"

    """Limit Away Preset."""
    LIMIT_AWAY_PRESET = "limit.away.preset"

    """Limit Away State."""
    LIMIT_AWAY_STATE = "limit.away.state"

    """Limit Away Triggered."""
    LIMIT_AWAY_TRIGGERED = "limit.away.triggered"

    """Limit Away Type."""
    LIMIT_AWAY_TYPE = "limit.away.type"

    """Limit C Action."""
    LIMIT_C_ACTION = "limit.c.action"

    """Limit C Edge."""
    LIMIT_C_EDGE = "limit.c.edge"

    """Limit C Pos."""
    LIMIT_C_POS = "limit.c.pos"

    """Limit C Posupdate."""
    LIMIT_C_POSUPDATE = "limit.c.posupdate"

    """Limit C Preset."""
    LIMIT_C_PRESET = "limit.c.preset"

    """Limit C State."""
    LIMIT_C_STATE = "limit.c.state"

    """Limit C Triggered."""
    LIMIT_C_TRIGGERED = "limit.c.triggered"

    """Limit C Type."""
    LIMIT_C_TYPE = "limit.c.type"

    """Limit Cycle Dist."""
    LIMIT_CYCLE_DIST = "limit.cycle.dist"

    """Limit D Action."""
    LIMIT_D_ACTION = "limit.d.action"

    """Limit D Edge."""
    LIMIT_D_EDGE = "limit.d.edge"

    """Limit D Pos."""
    LIMIT_D_POS = "limit.d.pos"

    """Limit D Posupdate."""
    LIMIT_D_POSUPDATE = "limit.d.posupdate"

    """Limit D Preset."""
    LIMIT_D_PRESET = "limit.d.preset"

    """Limit D State."""
    LIMIT_D_STATE = "limit.d.state"

    """Limit D Triggered."""
    LIMIT_D_TRIGGERED = "limit.d.triggered"

    """Limit D Type."""
    LIMIT_D_TYPE = "limit.d.type"

    """Limit Detect Decelonly."""
    LIMIT_DETECT_DECELONLY = "limit.detect.decelonly"

    """Limit Detect Maxspeed."""
    LIMIT_DETECT_MAXSPEED = "limit.detect.maxspeed"

    """Limit Home Action."""
    LIMIT_HOME_ACTION = "limit.home.action"

    """Limit Home Edge."""
    LIMIT_HOME_EDGE = "limit.home.edge"

    """Limit Home Pos."""
    LIMIT_HOME_POS = "limit.home.pos"

    """Limit Home Posupdate."""
    LIMIT_HOME_POSUPDATE = "limit.home.posupdate"

    """Limit Home Preset."""
    LIMIT_HOME_PRESET = "limit.home.preset"

    """Limit Home State."""
    LIMIT_HOME_STATE = "limit.home.state"

    """Limit Home Triggered."""
    LIMIT_HOME_TRIGGERED = "limit.home.triggered"

    """Limit Home Type."""
    LIMIT_HOME_TYPE = "limit.home.type"

    """Limit Max."""
    LIMIT_MAX = "limit.max"

    """Limit Min."""
    LIMIT_MIN = "limit.min"

    """Limit Start Pos."""
    LIMIT_START_POS = "limit.start.pos"

    """Limit Swapinputs."""
    LIMIT_SWAPINPUTS = "limit.swapinputs"

    """Lockstep Numgroups."""
    LOCKSTEP_NUMGROUPS = "lockstep.numgroups"

    """Lockstep Tolerance."""
    LOCKSTEP_TOLERANCE = "lockstep.tolerance"

    """Maxspeed."""
    MAXSPEED = "maxspeed"

    """Motion Accelonly."""
    MOTION_ACCELONLY = "motion.accelonly"

    """Motion Decelonly."""
    MOTION_DECELONLY = "motion.decelonly"

    """Motion Index Dist."""
    MOTION_INDEX_DIST = "motion.index.dist"

    """Motion Index Num."""
    MOTION_INDEX_NUM = "motion.index.num"

    """Motor Current Max."""
    MOTOR_CURRENT_MAX = "motor.current.max"

    """Parking State."""
    PARKING_STATE = "parking.state"

    """Peripheral Serial."""
    PERIPHERAL_SERIAL = "peripheral.serial"

    """Peripheralid."""
    PERIPHERALID = "peripheralid"

    """Pos."""
    POS = "pos"

    """Resolution."""
    RESOLUTION = "resolution"

    """Scope Delay."""
    SCOPE_DELAY = "scope.delay"

    """Scope Timebase."""
    SCOPE_TIMEBASE = "scope.timebase"

    """Stream Numbufs."""
    STREAM_NUMBUFS = "stream.numbufs"

    """Stream Numstreams."""
    STREAM_NUMSTREAMS = "stream.numstreams"

    """System Access."""
    SYSTEM_ACCESS = "system.access"

    """System Axiscount."""
    SYSTEM_AXISCOUNT = "system.axiscount"

    """System Current."""
    SYSTEM_CURRENT = "system.current"

    """System Led Enable."""
    SYSTEM_LED_ENABLE = "system.led.enable"

    """System Serial."""
    SYSTEM_SERIAL = "system.serial"

    """System Temperature."""
    SYSTEM_TEMPERATURE = "system.temperature"

    """System Voltage."""
    SYSTEM_VOLTAGE = "system.voltage"

    """Trigger Numactions."""
    TRIGGER_NUMACTIONS = "trigger.numactions"

    """Trigger Numtriggers."""
    TRIGGER_NUMTRIGGERS = "trigger.numtriggers"

    """Vel."""
    VEL = "vel"

    """Version."""
    VERSION = "version"

    """Version Build."""
    VERSION_BUILD = "version.build"

    """Virtual Numvirtual."""
    VIRTUAL_NUMVIRTUAL = "virtual.numvirtual"
